import sys, os, string, math, time, datetime
import argparse
import numpy

from LxiInstr import *

class Dp832PowerSupply(LxiInstr):

    known_list = [ "DP832" ]

    # Calibration points for channels 1 & 2
    cal_dacv12 = [0.2, 0.5, 1.2, 2, 3.2, 4.1, 5.2, 6.9, 7.5, 8.7, 10.1, 11.8, 12.6, 13.5,
                  15, 15.8, 16.5, 17.3, 18.5, 19.1, 19.9, 20.2, 20.8, 21.8, 22.4, 22.7,
                  23.9, 24.3, 25.7, 26.9, 27.9, 28.5, 28.9, 29.8, 30.2, 32];
    cal_adcv12 = [0, 0.05, 0.1, 0.5, 1, 5, 10, 12.8, 20, 30, 32];
    cal_daci12 = [0.1, 0.25, 0.5, 0.8, 1, 1.25, 1.5, 1.75, 1.9, 2.15, 2.35, 2.5, 2.75,
                  3, 3.2];
    cal_adci12 = [0, 0.01, 0.1, 1, 2, 3, 3.2];

    # Calibration points for channel 3
    cal_dacv3 = [0.1, 0.2, 0.4, 0.85, 1.2, 1.8, 2.55, 3.1, 3.4, 4.1, 4.5, 5, 5.3];
    cal_adcv3 = [0, 0.005, 0.01, 0.02, 0.05, 0.1, 0.5, 1, 3, 5, 5.3];
    cal_daci3 = [0.1, 0.5, 1, 1.25, 1.5, 1.75, 2, 2.25, 2.5, 2.75, 3, 3.2];
    cal_adci3 = [0, 0.1, 1, 2, 3, 3.2];

    # Calibration points, indexable by channel N (1:3)
    cal_dacv = [ [], cal_dacv12, cal_dacv12, cal_dacv3 ]
    cal_adcv = [ [], cal_adcv12, cal_adcv12, cal_adcv3 ]
    cal_daci = [ [], cal_daci12, cal_daci12, cal_daci3 ]
    cal_adci = [ [], cal_adci12, cal_adci12, cal_adci3 ]

    stabilization_time_sec = 2

    def __init__(self, addr):
        super(Dp832PowerSupply, self).__init__(addr)

        if self.model() not in self.known_list:
            raise ValueError("%s is not a DP832 power supply" % self.model())
        self._set = False
        self._dmm = None
        self._channels = xrange(1,4)   # [1-4)
        self._calib_nplc = 100

    def reset(self):
        self.cmd_reset()

    def self_test(self):
        r = self.query("SYSTem:SELF:TEST:BOARD?")
        if r != "PASS,PASS":
            raise ValueError("Self-test failed for %s" % self.model())

    def get_temp(self):
        temp_c = self.query_ascii_values("SYSTem:SELF:TEST:TEMP?")[0]
        temp_f = temp_c * 9.0/5.0 + 32.0
        return [temp_c, temp_f]

    def channel_on(self, channel):
        self.command("OUTPUT CH%d,ON" % channel)

    def channel_off(self, channel):
        self.command("OUTPUT CH%d,OFF" % channel)

    def emergency_off(self):
        # Ignore errors, if any
        self.send("OUTPUT CH1,OFF")
        self.send("OUTPUT CH2,OFF")
        self.send("OUTPUT CH3,OFF")


    # Calibration - this is very ad-hoc and DP832 specific.  Some of the
    # workflow end of things should probably be delegated, or this can perhaps
    # be a delegate for a Calibrator
    def set_calib_dmm(self, dmm):
        self._dmm = dmm

    def set_calib_nplc(self, nplc=100):
        self._calib_nplc = nplc

    def set_calib_channels(self, channels):
        self._channels = channels

    def print_val(self, what, unit, step, point, value):
        if point != 0:
            value_pct_err=math.fabs((point-value)/point)*100.0
        else:
            value_pct_err=0.0
        print "%5s %2d :  Point:%+6.4g%s - DMM:%+12.8g%s - Err:%4g%%" % (what, step, point, unit, value, unit, value_pct_err)

    def calib_single(self, what, value_table, channel, unit, index):
        print
        print "Running %s for channel %d" % (what, channel)
        values = value_table[channel]
        step   = 0
        ident = unit
        if unit == 'A':
            ident = 'C'

        for value in values:
            self.command("CALibration:Set CH%d,%s,%d,%g%s,%d" % (channel, ident, step, value, unit, index));
            time.sleep(self.stabilization_time_sec)
            value_read = self._dmm.read()
            self.command("CALibration:MEAS CH%d,%s,%d,%g%s,%d" % (channel, ident, step, value_read, unit, index));
            self.print_val(what, unit, step, value, value_read);
            step += 1

    def wait_for_enter(self, msg):
        print
        print msg
        raw_input("Press enter to continue")

    
    def perform_calibration(self, update=False):
        for channel in self._channels:
            print
            print "Starting calibration of channel %d" % channel
            self.command("CALibration:Start 11111,CH%d" % channel)
            self.command("CALibration:Clear CH%d,ALL" % channel)
            self.reset()

            # === voltage ===
            self.wait_for_enter("Connect the %s VOLTAGE inputs to the %s channel %d" % (self._dmm.model(), self.model(), channel))

            self._dmm.select_dcv(instr_range="AUTO")
            self._dmm.select_nplc(self._calib_nplc)

            self.channel_on(channel)

            self.calib_single("DAC-V", self.cal_dacv, channel, "V", 1)
            self.calib_single("ADC-V", self.cal_adcv, channel, "V", 0)

            self.channel_off(channel)

            # === current ===
            self.wait_for_enter("Connect the %s 10A CURRENT inputs to %s channel %d" %  (self._dmm.model(), self.model(), channel))

            self._dmm.select_dci(instr_range="10A")
            self._dmm.select_nplc(self._calib_nplc)
            self.channel_on(channel)

            calib_single("DAC-I", self.cal_daci, channel, "A", 1)
            calib_single("ADC-I", self.cal_adci, channel, "A", 0)

            self.channel_off(channel)

            # === update ===

            if update:
                print
                print "Updating calibration data for channel %d" % channel
                self.command("CALibration:End %s,CH%d" % (now, channel))
                print

            print
            print "Channel %d finished" % channel

        make_model = "%s %s" % (self.make(), self.model())

        if not update:
            print
            print "Instrument check of %s finished.  Rerun with --set to perform calibration." % make_model
        else:
            print
            print "Calibration of %s done.  Make sure to power cycle the unit, then go to the system cal/test menu and check the last calibration time.  It should be '%s'." % (make_model, now)
