import sys, os, string, math, time, datetime
import argparse
import numpy

from LxiInstr import *

# This should really implement an abstract DMM interface that can be
# used by anything that needs a DMM, but Python makes this
# ridiculously complicated for no benefit.  It still can't find problems
# like missing methods until it actually tries to execute the code (i.e.
# no early failures).  All you get in exchange is meta-induced headaches.

class KeysightTrueVoltDmm(LxiInstr):

    # XXX move DM3068 out of here into its own implementation of LxiInstr
    known_list = ["34461A", "34465A", "34470A", "DM3068"]
    have_temp_check = [ '34461A', '34465A', '34470A' ]
    init_cmds = { 'DM3068': 'CMDSet AGILENT' }

    def __init__(self, addr):
        super(KeysightTrueVoltDmm, self).__init__(addr)
        if self.model() not in self.known_list:
            raise ValueError("%s is not a known Keysight TrueVolt DMM" % self.model())
        self._multiread = 1

    def set_multiread(self, n):
        n = (int)(n)
        if n <= 0 or n >= 100:
            raise ValueError("Bad value for %s multiread: %d" % (self.model(), n))
        self._multiread = n

    def read(self):
        sum = 0.0
        for multiread in range(0, self._multiread):
            values = self.query_ascii_values("READ?")
            if len(values) != 1:
                raise ValueError("Wrong number of values returned by %s" % self.model())
            reading = values[0]
            sum += reading
        return sum/(float)(self._multiread);
    
    def self_test(self):
        r = self.query("*TST?")
        if r != '+0':
            raise ValueError("Self-test failed for %s" % self.model())

    def get_temp(self):
        if self.model() in self.have_temp_check:
            temp_c = self.query("SYST:TEMP?")
            temp_c = (float)(temp_c)
            temp_f = temp_c * 9.0/5.0 + 32.0
            return [temp_c, temp_f]

        return [None, None]

    def get_uptime(self):
        uptime = self.query("SYST:UPT?")
        (upd, uph, upm, ups) = uptime.split(',', 3)
        return [(int)(upd), (int)(uph), (int)(upm), (int)(ups)]

    def select_dcv(self, instr_range = "AUTO"):
        self.command("CONF:VOLT:DC %s" % instr_range)

    def select_dci(self, instr_range = "3A"):
        self.command("CONF:CURR:DC %s" % instr_range)

    def select_acv(self, instr_range = "AUTO"):
        self.command("CONF:VOLT:AC %s" % instr_range)

    def select_aci(self, instr_range = "3A"):
        self.command("CONF:CURR:AC %s" % instr_range)

    def select_nplc(self, nplc = 10):
        self.command("VOLT:DC:NPLC %d" % nplc)

    def reset(self):
        if self.cmd_reset():
            if self.model() in self.init_cmds:
                self.command(self.init_cmds[self.model()])
