# This was tested using:
#   NI-VISA kext and discovery on OS X 10.10
#   Python2.7
#   PyVISA
#   LAN connected Rigol DP832 and Keysight 34465A
#
# It's based on EEVblog user LaurentR's MATLAB script.
# I factored the LXI and DMM ends into separate files to allow reuse of these
# for other purposes, like data logging.
# The calibration stuff is very DP832 specific, and contained in Dp832PowerSupply.
# It can use any DMM that implements the same methods and works the same as
# KeysightTrueVoltDmm.  The latter retain LaurentR's logic to work with a Rigol DM3068,
# but this is untested.
#
# Please note that this is provided without any warranty, either implied or explicit.
# It's solely AS-IS, and you use it at your own risk.

import sys, os, string, math, time, datetime
import argparse
import numpy

from LxiInstr import *
from KeysightTrueVoltDmm import *
from Dp832PowerSupply import *

#
# requires Python2.7
#
if sys.version_info[0] != 2 or sys.version_info[1] < 7:
    print "This requires python version 2.7+ (but not 3.x)"
    sys.exit(1)

parser = argparse.ArgumentParser(description='Calibrate Rigol DP832 using an LXI capable DMM')
parser.add_argument('dp832_addr', help='Rigol DP832 to calibrate.  Can be VISA address, or an IPv4 address.')
parser.add_argument('dmm_addr', help='DMM to use for calibration.  Can be VISA address, or an IPv4 address.')
parser.add_argument('-s', '--set', action='store_const', const=1, default=0, help='Update calibration after run.  Otherwise will only print.')
parser.add_argument('-t', '--trace', action='store_const', const=1, default=0, help='Trace commands.')


args = parser.parse_args()
args = vars(args)

arg_channels= range(1,3+1)

arg_ps_addr = args['dp832_addr']
arg_dmm_addr= args['dmm_addr']
arg_set     = args['set'] == 1
arg_trace   = args['trace'] == 1

if not ':' in arg_ps_addr:
    arg_ps_addr = "TCPIP0::%s::INSTR" % arg_ps_addr

if not ':' in arg_dmm_addr:
    arg_dmm_addr = "TCPIP0::%s::INSTR" % arg_dmm_addr

LxiInstr.trace = arg_trace

try:
    dmm_instr = KeysightTrueVoltDmm(arg_dmm_addr)
    ps_instr  = Dp832PowerSupply(arg_ps_addr)

except Exception as e:
    print "Error initializing: %s" % str(e)
    sys.exit(127)

print "Using: "
print "  DMM   : %s" % dmm_instr.oneliner()
print "  DP832 : %s" % ps_instr.oneliner()
print

if arg_set:
    print "Power Supply instrument calibration WILL BE UPDATED"
else:
    print "Calibration data will NOT be updated -- this is only a check"

ps_instr.reset()
dmm_instr.reset()

# In case it rolls over while running, so the channels match
now = datetime.date.today().isoformat()

def self_test_and_temp(instr):
    print
    print "Running %s board self-test..." % instr.model()
    instr.self_test()
    print "%s self-test passed" % instr.model()

    (temp_c, temp_f) = instr.get_temp()
    if not temp_c is None and not temp_f is None:
        print  "%s internal temp: %gC (%gF)" % (instr.model(), temp_c, temp_f)


try:
    # === self test ===

    ps_instr.wait_for_enter("Remove all leads to perform self test")

    self_test_and_temp(ps_instr)
    self_test_and_temp(dmm_instr)

    (upd, uph, upm, ups) = dmm_instr.get_uptime()
    print "%s uptime: %dd %02u:%02u:%02u" % (dmm_instr.model(), upd, uph, upm, ups)

    # === calibration ===
    ps_instr.set_calib_dmm(dmm_instr)
    ps_instr.perform_calibration(arg_set)

except (Exception, KeyboardInterrupt) as e:
    # Error - turn off PSU outputs
    print "Shutting off PSU outputs"
    ps_instr.emergency_off()

    print "ERROR: %s" % str(e)
    print "Calibration failed. Terminating."
